#!/bin/bash 
msVersion="1.2"
# Script to execute a hyperscale masking job via continuous data hook
# Author: Tom Syth
#
# Dependencies: curl and jq executables

# status check interval in minutes
HS_STATUS_CHECK_INTERVAL=1

# If HS job hasn't completed in this number of hours, then try to cancel the job and abort this script
HS_MAX_RUNTIME=24

# Hyperscale API key
HS_API_KEY="1.jMc2POkefCQYqPG2TrM1JTkjy8xu0Dglc8uVxwZzrJixd3OBzydFQLjEfmndRayB"

# HS Orchestrator URL Base
URLBASE='https://10.160.1.140/api'

# Hyperscale Job ID to run
HS_JOB_ID=10

# full path to 'curl' executable plus options
CURL='/usr/bin/curl -k -s -X'

# full path to 'jq' executable
JQ='/usr/bin/jq'

# Logfile is overwritten for each run if name is not unique
LOGFILE="hs_${HS_JOB_ID}_"`date +%d%m%Y_%H%M%S`".log"

######################## Do not edit below this line ##########################

# write out log message
function write_log_msg()
{
        echo `date -u '+%d%m%Y:%H%M%S'` $* >> ${LOGFILE}
}

# check the status of the running job
function chk_status()
{
   if [ "${HS_STATUS}" = "RUNNING" ]; then
     HS_RESULT=`${CURL} GET ${URLBASE}'/executions/'${HS_EXECUTION_ID}'/summary' \
                     -H "Authorization: apk ${HS_API_KEY}" \
                     -H 'Accept: application/json'`
     HS_STATUS=`echo ${HS_RESULT} | ${JQ} '.status' | tr -d '"'`
     case "${HS_STATUS}" in
                 SUCCEEDED)
                     write_log_msg "Hyperscale job ${HS_JOB_ID} completed successfully"
                     echo ${HS_RESULT} | ${JQ} '.' >> ${LOGFILE}
                     exit 0
                     ;;
                 FAILED)
                     write_log_msg "Hyperscale job ${HS_JOB_ID} did not complete successfully"
                     # on failure, pull full report and dump to log
                     HS_RESULT=`${CURL} GET ${URLBASE}'/executions/'${HS_EXECUTION_ID}'?task=ALL&object_status=FAILED&object_details_status=FAILED' \
                     -H "Authorization: apk ${HS_API_KEY}" \
                     -H 'Accept: application/json'`
                     echo ${HS_RESULT} | ${JQ} '.' >> ${LOGFILE}
                     exit 1
                     ;;
                RUNNING)
                     write_log_msg "Job ${HS_JOB_ID} still running"
                     ;;
                      *)
                     # Unexpected response to status call - dump out entire response and reset status to RUNNING
                     # so check happens again next loop iteration.
                     write_log_msg "Unknown status ${HS_STATUS} returned for job ${HS_JOB_ID}"
                     echo ${HS_RESULT} | ${JQ} '.' >> ${LOGFILE}
                     HS_STATUS="RUNNING"
                     ;;
     esac
   fi
}

# Main line code starts here

# Convert MAX_RUN_TIME from hours to seconds
MAX_RUNTIME=$(( ${HS_MAX_RUNTIME} * 3600 ))

# Initialize log file for this run.
(
cat <<EOM
***** `date -u` : Hyperscale masking  script version ${msVersion} starting
Paramters:
  JOB_ID:             ${HS_JOB_ID}
  MAX EXECUTION TIME: ${HS_MAX_RUNTIME} hours
  LOGFILE:            ${LOGFILE}

EOM
) > ${LOGFILE}

# launch HS job
HS_RESPONSE=`${CURL} POST "${URLBASE}/executions" \
              -H 'accept: application/json' \
              -H "Authorization: apk ${HS_API_KEY}" \
              -H 'Content-Type: application/json' \
              -d "{ \"job_id\": ${HS_JOB_ID} }"`

# get Hyperscale execution ID if successfully started otherwize exit with error
HS_STATUS=`echo ${HS_RESPONSE} | ${JQ} '.status' | tr -d '"'`

# get Hyperscale job start time
HS_JOB_START_TIME=`date '+%s'`

if [ "${HS_STATUS}" = "RUNNING" ]; then 
  write_log_msg "Hyperscale job id ${HS_JOB_ID} started" 
  write_log_msg "API Response: ${HS_RESPONSE}" 
  HS_EXECUTION_ID=`echo ${HS_RESPONSE} | ${JQ} '.id'`
else 
  write_log_msg "Hyperscale job id ${HS_JOB_ID} failed to start" 
  write_log_msg "API Response: ${HS_RESPONSE}"
  exit 1
fi

# loop and check status until completed
while [ "${HS_STATUS}" = "RUNNING" ]; do
  # want to check current runtime against max and try to cancel the job if time has expired
  if [ $(( ${HS_JOB_START_TIME} + ${MAX_RUNTIME} )) -lt `date '+%s'` ];
  then
     write_log_msg "Hyperscale job id ${HS_JOB_ID} exceeded max runtime of ${HS_MAX_RUN_TIME} hours." 
     # try to issue a cancel against the running execution ID
     # HS_RESPONSE=`${CURL} PUT "${URLBASE}/executions/${HS_EXECUTION_ID}/cancel" \
     #                -H 'accept: application/json' \
     #                -H "Authorization: apk ${HS_API_KEY}"`
     #                -H 'Content-Type: application/json' \
     #write_log_msg "API Response: ${HS_RESPONSE}" 
     exit 1
  fi
  sleep $(( ${HS_STATUS_CHECK_INTERVAL} * 60 ))
  chk_status
done

